@@ -140,6 +140,7 @@ struct _GrDetailsPage
         GtkWidget *time_spin;
         GtkWidget *start_button;
         GtkWidget *favorite_button;
+        GtkWidget *shop_button;
         GtkWidget *duration_stack;
         GtkWidget *remaining_time_label;
         GtkWidget *chef_label;
@@ -152,6 +153,7 @@ struct _GrDetailsPage
         GtkWidget *error_revealer;
 
         guint save_timeout;
+        guint inhibit_cookie;
 
         char *uri;
 };
@@ -209,6 +211,11 @@ set_cooking (GrDetailsPage *page,
 {
         const char *id;
         CookingData *cd;
+        GtkWidget *window;
+        GtkApplication *app;
+
+        window = gtk_widget_get_ancestor (GTK_WIDGET (page), GTK_TYPE_APPLICATION_WINDOW);
+        app = gtk_window_get_application (GTK_WINDOW (window));
 
         id = gr_recipe_get_id (page->recipe);
 
@@ -229,6 +236,14 @@ set_cooking (GrDetailsPage *page,
                 g_signal_connect (cd->timer, "notify::remaining", G_CALLBACK (timer_remaining_changed), page);
                 timer_remaining_changed (cd->timer, NULL, page);
                 gtk_revealer_set_reveal_child (GTK_REVEALER (page->cooking_revealer), TRUE);
+
+                if (page->inhibit_cookie == 0) {
+                        page->inhibit_cookie = gtk_application_inhibit (app,
+                                                                        GTK_WINDOW (window),
+                                                                        GTK_APPLICATION_INHIBIT_SUSPEND |
+                                                                        GTK_APPLICATION_INHIBIT_IDLE,
+                                                                        _("Cooking"));
+                }
         }
         else {
                 if (cd) {
@@ -239,6 +254,11 @@ set_cooking (GrDetailsPage *page,
 
                 g_object_set (page->timer, "timer", NULL, NULL);
                 gtk_revealer_set_reveal_child (GTK_REVEALER (page->cooking_revealer), FALSE);
+
+                if (page->inhibit_cookie != 0) {
+                        gtk_application_uninhibit (app, page->inhibit_cookie);
+                        page->inhibit_cookie = 0;
+                }
         }
 }
 
@@ -246,8 +266,11 @@ static void
 delete_recipe (GrDetailsPage *page)
 {
         GrRecipeStore *store;
+        g_autoptr(GrRecipe) recipe = NULL;
         GtkWidget *window;
 
+        recipe = g_object_ref (page->recipe);
+
         store = gr_app_get_recipe_store (GR_APP (g_application_get_default ()));
         gr_recipe_store_remove_recipe (store, page->recipe);
         g_set_object (&page->recipe, NULL);
@@ -255,6 +278,8 @@ delete_recipe (GrDetailsPage *page)
 
         window = gtk_widget_get_ancestor (GTK_WIDGET (page), GTK_TYPE_APPLICATION_WINDOW);
         gr_window_go_back (GR_WINDOW (window));
+
+        gr_window_offer_undelete (GR_WINDOW (window), recipe);
 }
 
 static void
@@ -293,11 +318,6 @@ print_recipe (GrDetailsPage *page)
 static void
 export_recipe (GrDetailsPage *page)
 {
-        g_autofree char *dir = NULL;
-        g_autofree char *basename = NULL;
-        g_autofree char *path = NULL;
-        g_autoptr(GFile) file = NULL;
-
         if (!page->exporter) {
                 GtkWidget *window;
 
@@ -305,14 +325,7 @@ export_recipe (GrDetailsPage *page)
                 page->exporter = gr_recipe_exporter_new (GTK_WINDOW (window));
         }
 
-        dir = g_dir_make_tmp ("recipesXXXXXX", NULL);
-        basename = g_strconcat (gr_recipe_get_name (page->recipe), ".tar.gz", NULL);
-        if (dir)
-                path = g_build_filename (dir, basename, NULL);
-        else
-                path = g_strdup (basename);
-        file = g_file_new_for_path (path);
-        gr_recipe_exporter_export_to (page->exporter, page->recipe, file);
+        gr_recipe_exporter_export (page->exporter, page->recipe);
 }
 
 static void populate_ingredients (GrDetailsPage *page,
@@ -499,6 +512,18 @@ cook_it_later (GrDetailsPage *page)
                 gr_recipe_store_remove_favorite (store, page->recipe);
 }
 
+static void
+shop_it (GrDetailsPage *page)
+{
+        GrRecipeStore *store;
+
+        store = gr_app_get_recipe_store (GR_APP (g_application_get_default ()));
+        if (gtk_toggle_button_get_active (GTK_TOGGLE_BUTTON (page->shop_button)))
+                gr_recipe_store_add_to_shopping (store, page->recipe);
+        else
+                gr_recipe_store_remove_from_shopping (store, page->recipe);
+}
+
 static void
 details_page_finalize (GObject *object)
 {
@@ -718,6 +743,7 @@ gr_details_page_class_init (GrDetailsPageClass *klass)
         gtk_widget_class_bind_template_child (widget_class, GrDetailsPage, time_spin);
         gtk_widget_class_bind_template_child (widget_class, GrDetailsPage, start_button);
         gtk_widget_class_bind_template_child (widget_class, GrDetailsPage, favorite_button);
+        gtk_widget_class_bind_template_child (widget_class, GrDetailsPage, shop_button);
         gtk_widget_class_bind_template_child (widget_class, GrDetailsPage, duration_stack);
         gtk_widget_class_bind_template_child (widget_class, GrDetailsPage, remaining_time_label);
         gtk_widget_class_bind_template_child (widget_class, GrDetailsPage, chef_label);
@@ -740,6 +766,7 @@ gr_details_page_class_init (GrDetailsPageClass *klass)
         gtk_widget_class_bind_template_callback (widget_class, time_spin_output);
         gtk_widget_class_bind_template_callback (widget_class, check_clicked);
         gtk_widget_class_bind_template_callback (widget_class, cook_it_later);
+        gtk_widget_class_bind_template_callback (widget_class, shop_it);
         gtk_widget_class_bind_template_callback (widget_class, activate_link);
         gtk_widget_class_bind_template_callback (widget_class, dismiss_error);
 }
@@ -860,11 +887,13 @@ void
 gr_details_page_set_recipe (GrDetailsPage *page,
                             GrRecipe      *recipe)
 {
-        const char *id;
+        const char *id = NULL;
+        const char *old_id = NULL;
         const char *author;
         const char *prep_time;
         const char *cook_time;
         int serves;
+        int want_serves;
         const char *ingredients;
         const char *instructions;
         const char *notes;
@@ -875,10 +904,16 @@ gr_details_page_set_recipe (GrDetailsPage *page,
         g_autoptr(GArray) images = NULL;
         gboolean cooking;
         gboolean favorite;
+        gboolean shopping;
+        gboolean same_recipe;
+
+        if (page->recipe)
+                old_id = gr_recipe_get_id (page->recipe);
+        id = gr_recipe_get_id (recipe);
+        same_recipe = g_strcmp0 (old_id, id) == 0;
 
         g_set_object (&page->recipe, recipe);
 
-        id = gr_recipe_get_id (recipe);
         author = gr_recipe_get_author (recipe);
         serves = gr_recipe_get_serves (recipe);
         prep_time = gr_recipe_get_prep_time (recipe);
@@ -894,7 +929,12 @@ gr_details_page_set_recipe (GrDetailsPage *page,
         ing = gr_ingredients_list_new (ingredients);
         g_set_object (&page->ingredients, ing);
 
-        populate_ingredients (page, serves, serves);
+        if (same_recipe)
+                want_serves = gtk_spin_button_get_value_as_int (GTK_SPIN_BUTTON (page->serves_spin));
+        else
+                want_serves = serves;
+
+        populate_ingredients (page, want_serves, serves);
 
         if (prep_time[0] == '\0')
                 gtk_label_set_label (GTK_LABEL (page->prep_time_label), "");
@@ -908,7 +948,7 @@ gr_details_page_set_recipe (GrDetailsPage *page,
         gtk_label_set_label (GTK_LABEL (page->instructions_label), instructions);
         gtk_label_set_track_visited_links (GTK_LABEL (page->instructions_label), FALSE);
 
-        gtk_spin_button_set_value (GTK_SPIN_BUTTON (page->serves_spin), serves);
+        gtk_spin_button_set_value (GTK_SPIN_BUTTON (page->serves_spin), want_serves);
         gtk_widget_set_sensitive (page->serves_spin, ing != NULL);
 
         cooking = g_hash_table_lookup (page->cooking, id) != NULL;
@@ -919,6 +959,9 @@ gr_details_page_set_recipe (GrDetailsPage *page,
         favorite = gr_recipe_store_is_favorite (store, recipe);
         gtk_toggle_button_set_active (GTK_TOGGLE_BUTTON (page->favorite_button), favorite);
 
+        shopping = gr_recipe_store_is_in_shopping (store, recipe);
+        gtk_toggle_button_set_active (GTK_TOGGLE_BUTTON (page->shop_button), shopping);
+
         chef = gr_recipe_store_get_chef (store, author);
         g_set_object (&page->chef, chef);
 
@@ -969,9 +1012,12 @@ details_page_reload (GrDetailsPage *page,
         const char *name;
         const char *new_name;
 
+        if (recipe == NULL || page->recipe == NULL)
+                return;
+
         name = gr_recipe_get_id (page->recipe);
         new_name = gr_recipe_get_id (recipe);
-        if (strcmp (name, new_name) == 0)
+        if (g_strcmp0 (name, new_name) == 0)
                 gr_details_page_set_recipe (page, recipe);
 }
 
