@@ -927,8 +927,10 @@ recipe_row_activated (GtkListBox    *list,
         GtkTextIter start, end;
         GtkTextTag *tag;
         GdkRGBA color;
+        GtkStateFlags state = gtk_widget_get_state_flags (GTK_WIDGET (self->instructions_field));
+        GtkStyleContext *context = gtk_widget_get_style_context (GTK_WIDGET (self->instructions_field));
 
-        gdk_rgba_parse (&color, "blue");
+        gtk_style_context_get_color (context, state | GTK_STATE_FLAG_LINK, &color);
 
         recipe = GR_RECIPE (g_object_get_data (G_OBJECT (row), "recipe"));
         id = gr_recipe_get_id (recipe);
@@ -1305,62 +1307,62 @@ gr_edit_page_class_init (GrEditPageClass *klass)
 
         gtk_widget_class_set_template_from_resource (widget_class, "/org/gnome/Recipes/gr-edit-page.ui");
 
-        gtk_widget_class_bind_template_child (GTK_WIDGET_CLASS (klass), GrEditPage, error_revealer);
-        gtk_widget_class_bind_template_child (GTK_WIDGET_CLASS (klass), GrEditPage, error_label);
-        gtk_widget_class_bind_template_child (GTK_WIDGET_CLASS (klass), GrEditPage, name_label);
-        gtk_widget_class_bind_template_child (GTK_WIDGET_CLASS (klass), GrEditPage, name_entry);
-        gtk_widget_class_bind_template_child (GTK_WIDGET_CLASS (klass), GrEditPage, cuisine_combo);
-        gtk_widget_class_bind_template_child (GTK_WIDGET_CLASS (klass), GrEditPage, category_combo);
-        gtk_widget_class_bind_template_child (GTK_WIDGET_CLASS (klass), GrEditPage, season_combo);
-        gtk_widget_class_bind_template_child (GTK_WIDGET_CLASS (klass), GrEditPage, prep_time_combo);
-        gtk_widget_class_bind_template_child (GTK_WIDGET_CLASS (klass), GrEditPage, cook_time_combo);
-        gtk_widget_class_bind_template_child (GTK_WIDGET_CLASS (klass), GrEditPage, serves_spin);
-        gtk_widget_class_bind_template_child (GTK_WIDGET_CLASS (klass), GrEditPage, spiciness_combo);
-        gtk_widget_class_bind_template_child (GTK_WIDGET_CLASS (klass), GrEditPage, description_field);
-        gtk_widget_class_bind_template_child (GTK_WIDGET_CLASS (klass), GrEditPage, instructions_field);
-        gtk_widget_class_bind_template_child (GTK_WIDGET_CLASS (klass), GrEditPage, gluten_free_check);
-        gtk_widget_class_bind_template_child (GTK_WIDGET_CLASS (klass), GrEditPage, nut_free_check);
-        gtk_widget_class_bind_template_child (GTK_WIDGET_CLASS (klass), GrEditPage, vegan_check);
-        gtk_widget_class_bind_template_child (GTK_WIDGET_CLASS (klass), GrEditPage, vegetarian_check);
-        gtk_widget_class_bind_template_child (GTK_WIDGET_CLASS (klass), GrEditPage, milk_free_check);
-        gtk_widget_class_bind_template_child (GTK_WIDGET_CLASS (klass), GrEditPage, images);
-        gtk_widget_class_bind_template_child (GTK_WIDGET_CLASS (klass), GrEditPage, add_image_button);
-        gtk_widget_class_bind_template_child (GTK_WIDGET_CLASS (klass), GrEditPage, remove_image_button);
-        gtk_widget_class_bind_template_child (GTK_WIDGET_CLASS (klass), GrEditPage, rotate_image_left_button);
-        gtk_widget_class_bind_template_child (GTK_WIDGET_CLASS (klass), GrEditPage, rotate_image_right_button);
-        gtk_widget_class_bind_template_child (GTK_WIDGET_CLASS (klass), GrEditPage, author_label);
-        gtk_widget_class_bind_template_child (GTK_WIDGET_CLASS (klass), GrEditPage, ingredients_box);
-        gtk_widget_class_bind_template_child (GTK_WIDGET_CLASS (klass), GrEditPage, ingredient_popover);
-        gtk_widget_class_bind_template_child (GTK_WIDGET_CLASS (klass), GrEditPage, new_ingredient_name);
-        gtk_widget_class_bind_template_child (GTK_WIDGET_CLASS (klass), GrEditPage, new_ingredient_amount);
-        gtk_widget_class_bind_template_child (GTK_WIDGET_CLASS (klass), GrEditPage, new_ingredient_unit);
-        gtk_widget_class_bind_template_child (GTK_WIDGET_CLASS (klass), GrEditPage, new_ingredient_add_button);
-        gtk_widget_class_bind_template_child (GTK_WIDGET_CLASS (klass), GrEditPage, ing_list);
-        gtk_widget_class_bind_template_child (GTK_WIDGET_CLASS (klass), GrEditPage, ing_search_button);
-        gtk_widget_class_bind_template_child (GTK_WIDGET_CLASS (klass), GrEditPage, ing_search_button_label);
-        gtk_widget_class_bind_template_child (GTK_WIDGET_CLASS (klass), GrEditPage, ing_search_revealer);
+        gtk_widget_class_bind_template_child (widget_class, GrEditPage, error_revealer);
+        gtk_widget_class_bind_template_child (widget_class, GrEditPage, error_label);
+        gtk_widget_class_bind_template_child (widget_class, GrEditPage, name_label);
+        gtk_widget_class_bind_template_child (widget_class, GrEditPage, name_entry);
+        gtk_widget_class_bind_template_child (widget_class, GrEditPage, cuisine_combo);
+        gtk_widget_class_bind_template_child (widget_class, GrEditPage, category_combo);
+        gtk_widget_class_bind_template_child (widget_class, GrEditPage, season_combo);
+        gtk_widget_class_bind_template_child (widget_class, GrEditPage, prep_time_combo);
+        gtk_widget_class_bind_template_child (widget_class, GrEditPage, cook_time_combo);
+        gtk_widget_class_bind_template_child (widget_class, GrEditPage, serves_spin);
+        gtk_widget_class_bind_template_child (widget_class, GrEditPage, spiciness_combo);
+        gtk_widget_class_bind_template_child (widget_class, GrEditPage, description_field);
+        gtk_widget_class_bind_template_child (widget_class, GrEditPage, instructions_field);
+        gtk_widget_class_bind_template_child (widget_class, GrEditPage, gluten_free_check);
+        gtk_widget_class_bind_template_child (widget_class, GrEditPage, nut_free_check);
+        gtk_widget_class_bind_template_child (widget_class, GrEditPage, vegan_check);
+        gtk_widget_class_bind_template_child (widget_class, GrEditPage, vegetarian_check);
+        gtk_widget_class_bind_template_child (widget_class, GrEditPage, milk_free_check);
+        gtk_widget_class_bind_template_child (widget_class, GrEditPage, images);
+        gtk_widget_class_bind_template_child (widget_class, GrEditPage, add_image_button);
+        gtk_widget_class_bind_template_child (widget_class, GrEditPage, remove_image_button);
+        gtk_widget_class_bind_template_child (widget_class, GrEditPage, rotate_image_left_button);
+        gtk_widget_class_bind_template_child (widget_class, GrEditPage, rotate_image_right_button);
+        gtk_widget_class_bind_template_child (widget_class, GrEditPage, author_label);
+        gtk_widget_class_bind_template_child (widget_class, GrEditPage, ingredients_box);
+        gtk_widget_class_bind_template_child (widget_class, GrEditPage, ingredient_popover);
+        gtk_widget_class_bind_template_child (widget_class, GrEditPage, new_ingredient_name);
+        gtk_widget_class_bind_template_child (widget_class, GrEditPage, new_ingredient_amount);
+        gtk_widget_class_bind_template_child (widget_class, GrEditPage, new_ingredient_unit);
+        gtk_widget_class_bind_template_child (widget_class, GrEditPage, new_ingredient_add_button);
+        gtk_widget_class_bind_template_child (widget_class, GrEditPage, ing_list);
+        gtk_widget_class_bind_template_child (widget_class, GrEditPage, ing_search_button);
+        gtk_widget_class_bind_template_child (widget_class, GrEditPage, ing_search_button_label);
+        gtk_widget_class_bind_template_child (widget_class, GrEditPage, ing_search_revealer);
 
-        gtk_widget_class_bind_template_child (GTK_WIDGET_CLASS (klass), GrEditPage, unit_list);
-        gtk_widget_class_bind_template_child (GTK_WIDGET_CLASS (klass), GrEditPage, amount_search_button);
-        gtk_widget_class_bind_template_child (GTK_WIDGET_CLASS (klass), GrEditPage, amount_search_button_label);
-        gtk_widget_class_bind_template_child (GTK_WIDGET_CLASS (klass), GrEditPage, amount_search_revealer);
-        gtk_widget_class_bind_template_child (GTK_WIDGET_CLASS (klass), GrEditPage, recipe_popover);
-        gtk_widget_class_bind_template_child (GTK_WIDGET_CLASS (klass), GrEditPage, recipe_list);
-        gtk_widget_class_bind_template_child (GTK_WIDGET_CLASS (klass), GrEditPage, recipe_filter_entry);
-        gtk_widget_class_bind_template_child (GTK_WIDGET_CLASS (klass), GrEditPage, add_recipe_button);
-        gtk_widget_class_bind_template_child (GTK_WIDGET_CLASS (klass), GrEditPage, link_image_button);
-        gtk_widget_class_bind_template_child (GTK_WIDGET_CLASS (klass), GrEditPage, image_popover);
-        gtk_widget_class_bind_template_child (GTK_WIDGET_CLASS (klass), GrEditPage, image_flowbox);
+        gtk_widget_class_bind_template_child (widget_class, GrEditPage, unit_list);
+        gtk_widget_class_bind_template_child (widget_class, GrEditPage, amount_search_button);
+        gtk_widget_class_bind_template_child (widget_class, GrEditPage, amount_search_button_label);
+        gtk_widget_class_bind_template_child (widget_class, GrEditPage, amount_search_revealer);
+        gtk_widget_class_bind_template_child (widget_class, GrEditPage, recipe_popover);
+        gtk_widget_class_bind_template_child (widget_class, GrEditPage, recipe_list);
+        gtk_widget_class_bind_template_child (widget_class, GrEditPage, recipe_filter_entry);
+        gtk_widget_class_bind_template_child (widget_class, GrEditPage, add_recipe_button);
+        gtk_widget_class_bind_template_child (widget_class, GrEditPage, link_image_button);
+        gtk_widget_class_bind_template_child (widget_class, GrEditPage, image_popover);
+        gtk_widget_class_bind_template_child (widget_class, GrEditPage, image_flowbox);
 
-        gtk_widget_class_bind_template_callback (GTK_WIDGET_CLASS (klass), dismiss_error);
-        gtk_widget_class_bind_template_callback (GTK_WIDGET_CLASS (klass), add_image);
-        gtk_widget_class_bind_template_callback (GTK_WIDGET_CLASS (klass), remove_image);
-        gtk_widget_class_bind_template_callback (GTK_WIDGET_CLASS (klass), rotate_image_left);
-        gtk_widget_class_bind_template_callback (GTK_WIDGET_CLASS (klass), rotate_image_right);
-        gtk_widget_class_bind_template_callback (GTK_WIDGET_CLASS (klass), images_changed);
-        gtk_widget_class_bind_template_callback (GTK_WIDGET_CLASS (klass), add_ingredient2);
-        gtk_widget_class_bind_template_callback (GTK_WIDGET_CLASS (klass), remove_ingredient);
-        gtk_widget_class_bind_template_callback (GTK_WIDGET_CLASS (klass), ingredient_changed);
+        gtk_widget_class_bind_template_callback (widget_class, dismiss_error);
+        gtk_widget_class_bind_template_callback (widget_class, add_image);
+        gtk_widget_class_bind_template_callback (widget_class, remove_image);
+        gtk_widget_class_bind_template_callback (widget_class, rotate_image_left);
+        gtk_widget_class_bind_template_callback (widget_class, rotate_image_right);
+        gtk_widget_class_bind_template_callback (widget_class, images_changed);
+        gtk_widget_class_bind_template_callback (widget_class, add_ingredient2);
+        gtk_widget_class_bind_template_callback (widget_class, remove_ingredient);
+        gtk_widget_class_bind_template_callback (widget_class, ingredient_changed);
 
         gtk_widget_class_bind_template_callback (widget_class, ing_filter_changed);
         gtk_widget_class_bind_template_callback (widget_class, ing_filter_stop);
@@ -2186,6 +2188,11 @@ gr_edit_page_save (GrEditPage *page)
                               NULL);
                 ret = gr_recipe_store_update_recipe (store, page->recipe, old_id, &error);
         }
+        else if (name == NULL || name[0] == '\0') {
+                g_set_error (&error, G_IO_ERROR, G_IO_ERROR_FAILED,
+                             _("You need to provide a name for the recipe"));
+                ret = FALSE;
+        }
         else {
                 g_autoptr(GrRecipe) recipe = NULL;
                 const char *author;
@@ -2215,12 +2222,9 @@ gr_edit_page_save (GrEditPage *page)
                 g_set_object (&page->recipe, recipe);
         }
 
-        if (!ret)
-                goto error;
+        if (ret)
+          return TRUE;
 
-        return TRUE;
-
-error:
         gtk_label_set_label (GTK_LABEL (page->error_label), error->message);
         gtk_revealer_set_reveal_child (GTK_REVEALER (page->error_revealer), TRUE);
 
