@@ -440,6 +440,7 @@ gr_image_viewer_set_images (GrImageViewer *viewer,
         populate_preview (viewer);
         viewer->index = 0;
         set_current_image (viewer);
+        hide_controls (viewer);
 
         g_object_thaw_notify (G_OBJECT (viewer));
 }
@@ -476,6 +477,20 @@ open_filechooser (GrImageViewer *viewer)
         g_autoptr(GtkFileFilter) filter = NULL;
 
         window = gtk_widget_get_ancestor (GTK_WIDGET (viewer), GTK_TYPE_APPLICATION_WINDOW);
+
+        if (in_flatpak_sandbox () && !portals_available ()) {
+                GtkWidget *dialog;
+
+                dialog = gtk_message_dialog_new (GTK_WINDOW (window),
+                                                 GTK_DIALOG_MODAL|GTK_DIALOG_DESTROY_WITH_PARENT,
+                                                 GTK_MESSAGE_ERROR,
+                                                 GTK_BUTTONS_OK,
+                                                 _("Missing the desktop portals needed to open files from inside a flatpak sandbox. Please install xdg-desktop-portal and xdg-desktop-portal-gtk on your system."));
+                g_signal_connect (dialog, "response", G_CALLBACK (gtk_widget_destroy), NULL);
+                gtk_widget_show (dialog);
+                return;
+        }
+
         chooser = gtk_file_chooser_native_new (_("Select an Image"),
                                                GTK_WINDOW (window),
                                                GTK_FILE_CHOOSER_ACTION_OPEN,
