@@ -22,6 +22,7 @@
 #include "config.h"
 
 #include <glib/gi18n.h>
+#include <glib/gstdio.h>
 #ifdef ENABLE_AUTOAR
 #include <gnome-autoar/gnome-autoar.h>
 #include "glnx-shutil.h"
@@ -40,7 +41,8 @@ struct _GrRecipeExporter
 {
         GObject parent_instance;
 
-        GrRecipe *recipe;
+        GList *recipes;
+        GtkWidget *button_now;
         GtkWindow *window;
 
 #ifdef ENABLE_AUTOAR
@@ -50,6 +52,8 @@ struct _GrRecipeExporter
         GFile *output;
         GList *sources;
         char *dir;
+
+        GtkWidget *dialog_heading;
 };
 
 G_DEFINE_TYPE (GrRecipeExporter, gr_recipe_exporter, G_TYPE_OBJECT)
@@ -59,7 +63,7 @@ gr_recipe_exporter_finalize (GObject *object)
 {
         GrRecipeExporter *exporter = GR_RECIPE_EXPORTER (object);
 
-        g_clear_object (&exporter->recipe);
+        g_list_free_full (exporter->recipes, g_object_unref);
         g_clear_object (&exporter->dest);
         g_clear_object (&exporter->output);
         g_list_free_full (exporter->sources, g_object_unref);
@@ -106,7 +110,8 @@ cleanup_export (GrRecipeExporter *exporter)
 #endif
 
         g_clear_pointer (&exporter->dir, g_free);
-        g_clear_object (&exporter->recipe);
+        g_list_free_full (exporter->recipes, g_object_unref);
+        exporter->recipes = NULL;
         g_clear_object (&exporter->output);
         g_clear_object (&exporter->dest);
         g_list_free_full (exporter->sources, g_object_unref);
@@ -120,17 +125,22 @@ completed_cb (AutoarCompressor *compressor,
 {
         GtkWidget *dialog;
         g_autofree char *path =  NULL;
+        int n;
 
+        n = g_list_length (exporter->recipes);
         path = g_file_get_path (exporter->dest);
-        g_message (_("The recipe %s has been exported as “%s”"),
-                   gr_recipe_get_name (exporter->recipe), path);
+
+        g_message (ngettext ("%d recipe has been exported as “%s”",
+                             "%d recipes have been exported as “%s”", n),
+                   n, path);
 
         dialog = gtk_message_dialog_new (exporter->window,
                                          GTK_DIALOG_MODAL|GTK_DIALOG_DESTROY_WITH_PARENT,
                                          GTK_MESSAGE_INFO,
                                          GTK_BUTTONS_OK,
-                                         _("The recipe %s has been exported as “%s”"),
-                                         gr_recipe_get_name (exporter->recipe), path);
+                                         ngettext ("%d recipe has been exported as “%s”",
+                                                   "%d recipes have been exported as “%s”", n),
+                                         n, path);
         g_signal_connect (dialog, "response", G_CALLBACK (gtk_widget_destroy), NULL);
         gtk_widget_show (dialog);
 
@@ -147,17 +157,11 @@ decide_dest_cb (AutoarCompressor *compressor,
 #endif
 
 static gboolean
-prepare_export (GrRecipeExporter  *exporter,
-                GError           **error)
+export_one_recipe (GrRecipeExporter  *exporter,
+                   GrRecipe          *recipe,
+                   GKeyFile          *keyfile,
+                   GError           **error)
 {
-#ifndef ENABLE_AUTOAR
-        g_set_error (error, G_IO_ERROR, G_IO_ERROR_FAILED,
-                     _("This build does not support exporting"));
-
-        return FALSE;
-#else
-        g_autofree char *path = NULL;
-        g_autoptr(GKeyFile) keyfile = NULL;
         const char *key;
         const char *name;
         const char *author;
@@ -170,45 +174,35 @@ prepare_export (GrRecipeExporter  *exporter,
         const char *ingredients;
         const char *instructions;
         const char *notes;
-        const char *fullname;
-        const char *image_path;
         int serves;
         GrDiets diets;
         GDateTime *ctime;
         GDateTime *mtime;
         g_autoptr(GrChef) chef = NULL;
-        GrRecipeStore *store;
         g_autoptr(GArray) images = NULL;
         g_auto(GStrv) paths = NULL;
         g_autofree int *angles = NULL;
         g_autofree gboolean *dark = NULL;
         int i, j;
 
-        g_assert (exporter->dir == NULL);
-        g_assert (exporter->sources == NULL);
+        key = gr_recipe_get_id (recipe);
+        name = gr_recipe_get_name (recipe);
+        author = gr_recipe_get_author (recipe);
+        description = gr_recipe_get_description (recipe);
+        serves = gr_recipe_get_serves (recipe);
+        cuisine = gr_recipe_get_cuisine (recipe);
+        season = gr_recipe_get_season (recipe);
+        category = gr_recipe_get_category (recipe);
+        prep_time = gr_recipe_get_prep_time (recipe);
+        cook_time = gr_recipe_get_cook_time (recipe);
+        diets = gr_recipe_get_diets (recipe);
+        ingredients = gr_recipe_get_ingredients (recipe);
+        instructions = gr_recipe_get_instructions (recipe);
+        notes = gr_recipe_get_notes (recipe);
+        ctime = gr_recipe_get_ctime (recipe);
+        mtime = gr_recipe_get_mtime (recipe);
 
-        exporter->dir = g_mkdtemp (g_build_filename (g_get_tmp_dir (), "recipeXXXXXX", NULL));
-        path = g_build_filename (exporter->dir, "recipes.db", NULL);
-        keyfile = g_key_file_new ();
-
-        key = gr_recipe_get_id (exporter->recipe);
-        name = gr_recipe_get_name (exporter->recipe);
-        author = gr_recipe_get_author (exporter->recipe);
-        description = gr_recipe_get_description (exporter->recipe);
-        serves = gr_recipe_get_serves (exporter->recipe);
-        cuisine = gr_recipe_get_cuisine (exporter->recipe);
-        season = gr_recipe_get_season (exporter->recipe);
-        category = gr_recipe_get_category (exporter->recipe);
-        prep_time = gr_recipe_get_prep_time (exporter->recipe);
-        cook_time = gr_recipe_get_cook_time (exporter->recipe);
-        diets = gr_recipe_get_diets (exporter->recipe);
-        ingredients = gr_recipe_get_ingredients (exporter->recipe);
-        instructions = gr_recipe_get_instructions (exporter->recipe);
-        notes = gr_recipe_get_notes (exporter->recipe);
-        ctime = gr_recipe_get_ctime (exporter->recipe);
-        mtime = gr_recipe_get_mtime (exporter->recipe);
-
-        g_object_get (exporter->recipe, "images", &images, NULL);
+        g_object_get (recipe, "images", &images, NULL);
         paths = g_new0 (char *, images->len + 1);
         angles = g_new0 (int, images->len + 1);
         dark = g_new0 (gboolean, images->len + 1);
@@ -266,29 +260,27 @@ prepare_export (GrRecipeExporter  *exporter,
                 g_key_file_set_string (keyfile, key, "Modified", modified);
         }
 
-        if (!g_key_file_save_to_file (keyfile, path, error)) {
-                return FALSE;
-        }
+        return TRUE;
+}
 
-        exporter->sources = g_list_append (exporter->sources, g_file_new_for_path (path));
-
-        store = gr_app_get_recipe_store (GR_APP (g_application_get_default ()));
-        chef = gr_recipe_store_get_chef (store, author);
-        if (!chef)
-                return TRUE;
-
-        g_clear_pointer (&path, g_free);
-        g_clear_pointer (&keyfile, g_key_file_unref);
-
-        path = g_build_filename (exporter->dir, "chefs.db", NULL);
-        keyfile = g_key_file_new ();
+static gboolean
+export_one_chef (GrRecipeExporter  *exporter,
+                 GrChef            *chef,
+                 GKeyFile          *keyfile,
+                 GError           **error)
+{
+        const char *key;
+        const char *name;
+        const char *fullname;
+        const char *description;
+        const char *image_path;
 
         key = gr_chef_get_id (chef);
         name = gr_chef_get_name (chef);
         fullname = gr_chef_get_fullname (chef);
         description = gr_chef_get_description (chef);
         image_path = gr_chef_get_image (chef);
-        if (image_path) {
+        if (image_path && image_path[0]) {
                 g_autoptr(GFile) source = NULL;
                 g_autoptr(GFile) dest = NULL;
                 g_autofree char *basename = NULL;
@@ -313,11 +305,78 @@ prepare_export (GrRecipeExporter  *exporter,
         g_key_file_set_string (keyfile, key, "Fullname", fullname ? fullname : "");
         g_key_file_set_string (keyfile, key, "Description", description ? description : "");
 
-        if (!g_key_file_save_to_file (keyfile, path, error)) {
-                return FALSE;
+        return TRUE;
+}
+
+static gboolean
+prepare_export (GrRecipeExporter  *exporter,
+                GError           **error)
+{
+#ifndef ENABLE_AUTOAR
+        g_set_error (error, G_IO_ERROR, G_IO_ERROR_FAILED,
+                     _("This build does not support exporting"));
+
+        return FALSE;
+#else
+        g_autofree char *path = NULL;
+        g_autoptr(GKeyFile) keyfile = NULL;
+        GrRecipeStore *store;
+        GList *l;
+        g_autoptr(GHashTable) chefs = NULL;
+
+        store = gr_app_get_recipe_store (GR_APP (g_application_get_default ()));
+
+        g_assert (exporter->dir == NULL);
+        g_assert (exporter->sources == NULL);
+
+        exporter->dir = g_mkdtemp (g_build_filename (g_get_tmp_dir (), "recipeXXXXXX", NULL));
+
+        path = g_build_filename (exporter->dir, "recipes.db", NULL);
+        keyfile = g_key_file_new ();
+
+        for (l = exporter->recipes; l; l = l->next) {
+                GrRecipe *recipe = l->data;
+
+                if (!export_one_recipe (exporter, recipe, keyfile, error))
+                        return FALSE;
         }
 
+        if (!g_key_file_save_to_file (keyfile, path, error))
+                return FALSE;
+
         exporter->sources = g_list_append (exporter->sources, g_file_new_for_path (path));
+
+        g_clear_pointer (&path, g_free);
+        g_clear_pointer (&keyfile, g_key_file_unref);
+
+        path = g_build_filename (exporter->dir, "chefs.db", NULL);
+        keyfile = g_key_file_new ();
+
+        chefs = g_hash_table_new (g_str_hash, g_str_equal);
+        for (l = exporter->recipes; l; l = l->next) {
+                GrRecipe *recipe = l->data;
+                const char *author;
+                g_autoptr(GrChef) chef = NULL;
+
+                author = gr_recipe_get_author (recipe);
+                if (g_hash_table_contains (chefs, author))
+                        continue;
+
+                chef = gr_recipe_store_get_chef (store, author);
+                if (!chef)
+                        continue;
+
+                if (!export_one_chef (exporter, chef, keyfile, error))
+                        return FALSE;
+
+                g_hash_table_add (chefs, (gpointer)author);
+        }
+
+        if (!g_key_file_save_to_file (keyfile, path, error))
+                return FALSE;
+
+        exporter->sources = g_list_append (exporter->sources, g_file_new_for_path (path));
+
         return TRUE;
 #endif
 }
@@ -333,8 +392,7 @@ error_cb (gpointer          compressor,
                                          GTK_DIALOG_MODAL|GTK_DIALOG_DESTROY_WITH_PARENT,
                                          GTK_MESSAGE_ERROR,
                                          GTK_BUTTONS_OK,
-                                         _("Error while exporting recipe %s:\n%s"),
-                                         gr_recipe_get_name (exporter->recipe),
+                                         _("Error while exporting:\n%s"),
                                          error->message);
         g_signal_connect (dialog, "response", G_CALLBACK (gtk_widget_destroy), NULL);
         gtk_widget_show (dialog);
@@ -342,16 +400,11 @@ error_cb (gpointer          compressor,
         cleanup_export (exporter);
 }
 
-void
-gr_recipe_exporter_export_to (GrRecipeExporter *exporter,
-                              GrRecipe         *recipe,
-                              GFile            *file)
+static void
+start_export (GrRecipeExporter *exporter)
 {
         g_autoptr(GError) error = NULL;
 
-        g_set_object (&exporter->recipe, recipe);
-        g_set_object (&exporter->output, file);
-
         if (!prepare_export (exporter, &error)) {
                 error_cb (NULL, error, exporter);
                 return;
@@ -368,3 +421,209 @@ gr_recipe_exporter_export_to (GrRecipeExporter *exporter,
         autoar_compressor_start_async (exporter->compressor, NULL);
 #endif
 }
+
+static void
+export_dialog_response (GtkWidget        *dialog,
+                        int               response_id,
+                        GrRecipeExporter *exporter)
+{
+        if (response_id == GTK_RESPONSE_CANCEL) {
+                g_message ("not exporting now");
+        }
+        else if (response_id == GTK_RESPONSE_OK) {
+                g_autofree char *path = NULL;
+                int i;
+
+                g_message ("exporting %d recipes now", g_list_length (exporter->recipes));
+
+                for (i = 0; i < 1000; i++) {
+                        g_autofree char *tmp;
+                        tmp = g_strdup_printf ("%s/.var/app/org.gnome.Recipes/recipes%d.tar.gz", g_get_home_dir (), i);
+                        if (!g_file_test (tmp, G_FILE_TEST_EXISTS)) {
+                                path = g_strdup (tmp);
+                                break;
+                        }
+                }
+
+                if (!path) {
+                        g_autofree char *dir = NULL;
+
+                        dir = g_dir_make_tmp ("recipesXXXXXX", NULL);
+                        path = g_build_filename (dir, "recipes.tar.gz", NULL);
+                }
+
+                exporter->output = g_file_new_for_path (path);
+
+                start_export (exporter);
+        }
+
+        gtk_widget_destroy (dialog);
+        exporter->dialog_heading = NULL;
+}
+
+static void
+update_heading (GrRecipeExporter *exporter)
+{
+        g_autofree char *tmp = NULL;
+        int n;
+
+        n = g_list_length (exporter->recipes);
+        tmp = g_strdup_printf (ngettext ("%d recipe selected for export",
+                                         "%d recipes selected for export", n), n);
+        gtk_label_set_label (GTK_LABEL (exporter->dialog_heading), tmp);
+}
+
+static void
+update_export_button (GrRecipeExporter *exporter)
+{
+  gboolean empty = exporter->recipes == NULL;
+
+  if (empty)
+    gtk_widget_set_sensitive (exporter->button_now, FALSE);
+  else
+    gtk_widget_set_sensitive (exporter->button_now, TRUE);
+}
+
+static void
+row_activated (GtkListBox *list,
+               GtkListBoxRow *row,
+               GrRecipeExporter *exporter)
+{
+        GrRecipe *recipe;
+        GtkWidget *image;
+
+        recipe = GR_RECIPE (g_object_get_data (G_OBJECT (row), "recipe"));
+        image = GTK_WIDGET (g_object_get_data (G_OBJECT (row), "check"));
+
+        if (gtk_widget_get_opacity (image) > 0.5) {
+                exporter->recipes = g_list_remove (exporter->recipes, recipe);
+                g_object_unref (recipe);
+                gtk_widget_set_opacity (image, 0.0);
+        }
+        else {
+                exporter->recipes = g_list_append (exporter->recipes, g_object_ref (recipe));
+                gtk_widget_set_opacity (image, 1.0);
+        }
+
+        update_heading (exporter);
+        update_export_button (exporter);
+}
+
+static void
+add_recipe_row (GrRecipeExporter *exporter,
+                GtkWidget *list,
+                GrRecipe  *recipe)
+{
+        GtkWidget *row;
+        GtkWidget *box;
+        GtkWidget *label;
+        GtkWidget *image;
+
+        box = gtk_box_new (GTK_ORIENTATION_HORIZONTAL, 20);
+        gtk_widget_show (box);
+        g_object_set (box, "margin", 10, NULL);
+        label = gtk_label_new (gr_recipe_get_name (recipe));
+        gtk_label_set_xalign (GTK_LABEL (label), 0.0);
+        gtk_widget_show (label);
+        gtk_box_pack_start (GTK_BOX (box), label, TRUE, TRUE, 0);
+
+        image = gtk_image_new_from_icon_name ("object-select-symbolic", 1);
+        gtk_widget_show (image);
+        gtk_style_context_add_class (gtk_widget_get_style_context (image), "dim-label");
+        gtk_box_pack_start (GTK_BOX (box), image, FALSE, TRUE, 0);
+
+        row = gtk_list_box_row_new ();
+        gtk_widget_show (row);
+        gtk_container_add (GTK_CONTAINER (row), box);
+        g_object_set_data_full (G_OBJECT (row), "recipe", g_object_ref (recipe), g_object_unref);
+        g_object_set_data (G_OBJECT (row), "check", image);
+
+        gtk_container_add (GTK_CONTAINER (list), row);
+
+}
+
+static void
+all_headers (GtkListBoxRow *row,
+             GtkListBoxRow *before,
+             gpointer       user_data)
+{
+        GtkWidget *header;
+
+        header = gtk_list_box_row_get_header (row);
+        if (header)
+                return;
+
+        header = gtk_separator_new (GTK_ORIENTATION_HORIZONTAL);
+        gtk_list_box_row_set_header (row, header);
+}
+
+static int
+sort_recipe_row (GtkListBoxRow *row1,
+                 GtkListBoxRow *row2,
+                 gpointer       data)
+{
+        GrRecipe *r1, *r2;
+
+        r1 = GR_RECIPE (g_object_get_data (G_OBJECT (row1), "recipe"));
+        r2 = GR_RECIPE (g_object_get_data (G_OBJECT (row2), "recipe"));
+
+        return g_strcmp0 (gr_recipe_get_name (r1), gr_recipe_get_name (r2));
+}
+
+static void
+populate_recipe_list (GrRecipeExporter *exporter,
+                      GtkWidget        *list)
+{
+        GList *l;
+
+        gtk_list_box_set_header_func (GTK_LIST_BOX (list), all_headers, NULL, NULL);
+        gtk_list_box_set_sort_func (GTK_LIST_BOX (list), sort_recipe_row, NULL, NULL);
+
+        g_signal_connect (list, "row-activated", G_CALLBACK (row_activated), exporter);
+        for (l = exporter->recipes; l; l = l->next) {
+                GrRecipe *recipe = l->data;
+                add_recipe_row (exporter, list, recipe);
+        }
+}
+
+static void
+show_export_dialog (GrRecipeExporter *exporter)
+{
+        g_autoptr(GtkBuilder) builder = NULL;
+        GtkWidget *dialog;
+        GtkWidget *list;
+        g_autofree char *tmp = NULL;
+
+        builder = gtk_builder_new_from_resource ("/org/gnome/Recipes/recipe-export-dialog.ui");
+        dialog = GTK_WIDGET (gtk_builder_get_object (builder, "dialog"));
+        exporter->button_now = GTK_WIDGET (gtk_builder_get_object (builder, "button_now"));
+
+        gtk_window_set_transient_for (GTK_WINDOW (dialog), GTK_WINDOW (exporter->window));
+
+        list = GTK_WIDGET (gtk_builder_get_object (builder, "recipe_list"));
+        populate_recipe_list (exporter, list);
+
+        exporter->dialog_heading = GTK_WIDGET (gtk_builder_get_object (builder, "heading"));
+        update_heading (exporter);
+
+        g_signal_connect (dialog, "response", G_CALLBACK (export_dialog_response), exporter);
+        gtk_widget_show (dialog);
+}
+
+void
+gr_recipe_exporter_export (GrRecipeExporter *exporter,
+                           GrRecipe         *recipe)
+{
+        GList *l;
+
+        // TODO: listen for ::recipe-removed and filter out the list
+        for (l = exporter->recipes; l; l = l->next) {
+                if (l->data == (gpointer)recipe)
+                        goto dialog;
+        }
+
+        exporter->recipes = g_list_append (exporter->recipes, g_object_ref (recipe));
+
+dialog:
+        show_export_dialog (exporter);
+}
