@@ -25,6 +25,9 @@
 #include "gr-recipe-printer.h"
 #include "gr-ingredients-list.h"
 #include "gr-images.h"
+#include "gr-chef.h"
+#include "gr-recipe-store.h"
+#include "gr-app.h"
 #include "gr-utils.h"
 
 
@@ -67,6 +70,82 @@ gr_recipe_printer_new (GtkWindow *parent)
         return printer;
 }
 
+typedef struct {
+        GString *string;
+        gboolean ignore;
+} ParserData;
+
+static void
+start_element (GMarkupParseContext  *context,
+               const char           *element_name,
+               const char          **attribute_names,
+               const char          **attribute_values,
+               gpointer              user_data,
+               GError              **error)
+{
+        ParserData *data = user_data;
+
+        if (strcmp (element_name, "a") == 0)
+                data->ignore = TRUE;
+}
+
+static void
+end_element (GMarkupParseContext  *context,
+             const char           *element_name,
+             gpointer              user_data,
+             GError              **error)
+{
+        ParserData *data = user_data;
+
+        if (strcmp (element_name, "a") == 0)
+                data->ignore = FALSE;
+}
+
+static void
+text (GMarkupParseContext  *context,
+      const char           *text,
+      gsize                 text_len,
+      gpointer              user_data,
+      GError              **error)
+{
+        ParserData *data = user_data;
+
+        if (!data->ignore)
+                g_string_append_len (data->string, text, text_len);
+}
+
+static GMarkupParser parser = {
+       start_element,
+       end_element,
+       text,
+       NULL,
+       NULL,
+};
+
+static char *
+strip_links (const char *text)
+{
+        g_autoptr(GMarkupParseContext) context = NULL;
+        g_autoptr(GString) string = NULL;
+        g_autoptr(GError) error = NULL;
+        ParserData data;
+
+        string = g_string_new ("");
+        data.string = string;
+        data.ignore = FALSE;
+
+        context = g_markup_parse_context_new (&parser, 0, &data, NULL);
+
+        if (!g_markup_parse_context_parse (context, "<instructions>", -1, &error) ||
+            !g_markup_parse_context_parse (context, text, -1, &error) ||
+            !g_markup_parse_context_parse (context, "</instructions>", -1, &error)) {
+                g_message ("Failed to parse instructions: %s", error->message);
+                return g_strdup ("");
+        }
+
+        return g_strdup (string->str);
+}
+
 static void
 begin_print (GtkPrintOperation *operation,
              GtkPrintContext   *context,
@@ -92,6 +171,12 @@ begin_print (GtkPrintOperation *operation,
         PangoTabArray *tabs;
         g_autofree char **segs = NULL;
         g_auto(GStrv) ings = NULL;
+        g_autofree char *instructions = NULL;
+        g_autoptr(GrChef) chef = NULL;
+        GrRecipeStore *store;
+
+        store = gr_app_get_recipe_store (GR_APP (g_application_get_default ()));
+        chef = gr_recipe_store_get_chef (store, gr_recipe_get_author (printer->recipe));
 
         width = gtk_print_context_get_width (context);
         height = gtk_print_context_get_height (context);
@@ -102,10 +187,11 @@ begin_print (GtkPrintOperation *operation,
                 printer->image = load_pixbuf_fit_size (ri->path, ri->angle, width / 2, height / 4, FALSE);
         }
 
-        title_font = pango_font_description_from_string ("Cantarell Bold 24");
-        body_font = pango_font_description_from_string ("Cantarell 18");
+        title_font = pango_font_description_from_string ("Cantarell Bold 18");
+        body_font = pango_font_description_from_string ("Cantarell 12");
 
         printer->title_layout = gtk_print_context_create_pango_layout (context);
+        pango_layout_set_width (printer->title_layout, width * PANGO_SCALE);
         pango_layout_set_font_description (printer->title_layout, title_font);
 
         s = g_string_new ("");
@@ -120,7 +206,7 @@ begin_print (GtkPrintOperation *operation,
 
         g_string_truncate (s, 0);
 
-        g_string_append_printf (s, "%s %s\n", _("Author:"), gr_recipe_get_author (printer->recipe));
+        g_string_append_printf (s, "%s %s\n", _("Author:"), gr_chef_get_fullname (chef));
         g_string_append_printf (s, "%s %s\n", _("Preparation:"), gr_recipe_get_prep_time (printer->recipe));
         g_string_append_printf (s, "%s %s\n", _("Cooking:"), gr_recipe_get_cook_time (printer->recipe));
         g_string_append_printf (s, "%s %d\n", _("Serves:"), gr_recipe_get_serves (printer->recipe));
@@ -210,8 +296,10 @@ begin_print (GtkPrintOperation *operation,
         attr->end_index = s->len + 1;
         pango_attr_list_insert (attrs, attr);
 
+        instructions = strip_links (gr_recipe_get_translated_instructions (printer->recipe));
+
         g_string_append (s, "\n\n");
-        g_string_append (s, gr_recipe_get_translated_instructions (printer->recipe));
+        g_string_append (s, instructions);
 
         pango_layout_set_text (printer->bottom_layout, s->str, s->len);
         pango_layout_set_attributes (printer->bottom_layout, attrs);
@@ -398,6 +486,19 @@ gr_recipe_printer_print (GrRecipePrinter *printer,
 {
         GtkPrintOperation *operation;
 
+        if (in_flatpak_sandbox () && !portals_available ()) {
+                GtkWidget *dialog;
+
+                dialog = gtk_message_dialog_new (GTK_WINDOW (printer->window),
+                                                 GTK_DIALOG_MODAL|GTK_DIALOG_DESTROY_WITH_PARENT,
+                                                 GTK_MESSAGE_ERROR,
+                                                 GTK_BUTTONS_OK,
+                                                 _("Missing the desktop portals needed to print from inside a flatpak sandbox. Please install xdg-desktop-portal and xdg-desktop-portal-gtk on your system."));
+                g_signal_connect (dialog, "response", G_CALLBACK (gtk_widget_destroy), NULL);
+                gtk_widget_show (dialog);
+                return;
+        }
+
         printer->recipe = g_object_ref (recipe);
 
         operation = gtk_print_operation_new ();
