@@ -47,12 +47,18 @@ const char     *gr_recipe_get_ingredients  (GrRecipe   *recipe);
 const char     *gr_recipe_get_instructions (GrRecipe   *recipe);
 const char     *gr_recipe_get_notes        (GrRecipe   *recipe);
 gboolean        gr_recipe_contains_garlic  (GrRecipe   *recipe);
+gboolean        gr_recipe_contains_dairy   (GrRecipe   *recipe);
+gboolean        gr_recipe_contains_gluten  (GrRecipe   *recipe);
 int             gr_recipe_get_spiciness    (GrRecipe   *recipe);
 GDateTime      *gr_recipe_get_ctime        (GrRecipe   *recipe);
 GDateTime      *gr_recipe_get_mtime        (GrRecipe   *recipe);
 gboolean        gr_recipe_is_readonly      (GrRecipe   *recipe);
 
-gboolean        gr_recipe_matches          (GrRecipe   *recipe,
-                                            const char *term);
+const char     *gr_recipe_get_translated_name         (GrRecipe   *recipe);
+const char     *gr_recipe_get_translated_description  (GrRecipe   *recipe);
+const char     *gr_recipe_get_translated_instructions (GrRecipe   *recipe);
+
+gboolean        gr_recipe_matches          (GrRecipe    *recipe,
+                                            const char **terms);
 
 G_END_DECLS
