@@ -30,9 +30,9 @@
 #include "gr-list-page.h"
 #include "gr-cuisine-page.h"
 #include "gr-search-page.h"
+#include "gr-shopping-page.h"
 #include "gr-recipes-page.h"
 #include "gr-cuisines-page.h"
-#include "gr-ingredients-page.h"
 #include "gr-query-editor.h"
 #include "gr-recipe-importer.h"
 
@@ -53,10 +53,15 @@ struct _GrWindow
         GtkWidget *details_page;
         GtkWidget *edit_page;
         GtkWidget *list_page;
+        GtkWidget *chef_page;
+        GtkWidget *shopping_page;
         GtkWidget *search_page;
         GtkWidget *cuisines_page;
         GtkWidget *cuisine_page;
-        GtkWidget *ingredients_page;
+        GtkWidget *undo_revealer;
+        GtkWidget *undo_label;
+        GrRecipe  *undo_recipe;
+        guint undo_timeout_id;
 
         GrRecipeImporter *importer;
 
@@ -174,6 +179,19 @@ stop_search (GrWindow *window)
         gr_window_go_back (window);
 }
 
+static void
+search_mode_changed (GrWindow *window)
+{
+        const char *visible;
+
+        visible = gtk_stack_get_visible_child_name (GTK_STACK (window->main_stack));
+
+        if (!gtk_search_bar_get_search_mode (GTK_SEARCH_BAR (window->search_bar)) &&
+            strcmp (visible, "search") == 0) {
+                stop_search (window);
+        }
+}
+
 static void
 switch_to_search (GrWindow *window)
 {
@@ -280,11 +298,6 @@ window_keypress_handler (GtkWidget *widget,
         const char *visible;
 
         visible = gtk_stack_get_visible_child_name (GTK_STACK (window->main_stack));
-        if (strcmp (visible, "ingredients") == 0) {
-                gr_ingredients_page_scroll (GR_INGREDIENTS_PAGE (window->ingredients_page),
-                                            ((GdkEventKey*)event)->string);
-                return GDK_EVENT_STOP;
-        }
 
         if (strcmp (visible, "recipes") != 0 &&
             strcmp (visible, "cuisines") != 0 &&
@@ -312,7 +325,6 @@ window_mapped_handler (GtkWidget *widget)
 {
         GrWindow *window = GR_WINDOW (widget);
 
-        gtk_window_set_resizable (GTK_WINDOW (window), TRUE);
         gr_recipes_page_unexpand (GR_RECIPES_PAGE (window->recipes_page));
         gr_cuisines_page_unexpand (GR_CUISINES_PAGE (window->cuisines_page));
 }
@@ -343,9 +355,82 @@ gr_window_finalize (GObject *object)
 
         g_clear_object (&self->importer);
 
+        if (self->undo_timeout_id) {
+                g_source_remove (self->undo_timeout_id);
+                self->undo_timeout_id = 0;
+        }
+
         G_OBJECT_CLASS (gr_window_parent_class)->finalize (object);
 }
 
+static void
+close_undo (GrWindow *window)
+{
+        if (window->undo_timeout_id) {
+                g_source_remove (window->undo_timeout_id);
+                window->undo_timeout_id = 0;
+        }
+
+        g_clear_object (&window->undo_recipe);
+
+        gtk_revealer_set_reveal_child (GTK_REVEALER (window->undo_revealer), FALSE);
+}
+
+static void
+do_undo (GrWindow *window)
+{
+        GrRecipeStore *store;
+        g_autoptr(GrRecipe) recipe = NULL;
+
+        recipe = g_object_ref (window->undo_recipe);
+
+        store = gr_app_get_recipe_store (GR_APP (g_application_get_default ()));
+
+        gr_recipe_store_add_recipe (store, window->undo_recipe, NULL);
+        close_undo (window);
+
+        gr_window_show_recipe (window, recipe);
+}
+
+static gboolean
+undo_timeout (gpointer data)
+{
+        GrWindow *window = data;
+
+        close_undo (window);
+
+        return G_SOURCE_REMOVE;
+}
+
+void
+gr_window_offer_undelete (GrWindow *window,
+                          GrRecipe *recipe)
+{
+        g_autofree char *tmp = NULL;
+
+        g_set_object (&window->undo_recipe, recipe);
+        tmp = g_strdup_printf (_("Recipe “%s” deleted"), gr_recipe_get_name (recipe));
+        gtk_label_set_label (GTK_LABEL (window->undo_label), tmp);
+
+        gtk_revealer_set_reveal_child (GTK_REVEALER (window->undo_revealer), TRUE);
+        window->undo_timeout_id = g_timeout_add_seconds (10, undo_timeout, window);
+}
+
+static void
+shopping_title_changed (GrWindow *window)
+{
+        const char *page;
+
+        page = gtk_stack_get_visible_child_name (GTK_STACK (window->main_stack));
+
+        if (strcmp (page, "shopping") == 0) {
+                g_autofree char *title = NULL;
+
+                g_object_get (window->shopping_page, "title", &title, NULL);
+                gtk_window_set_title (GTK_WINDOW (window), title);
+        }
+}
+
 static void
 gr_window_class_init (GrWindowClass *klass)
 {
@@ -368,10 +453,13 @@ gr_window_class_init (GrWindowClass *klass)
         gtk_widget_class_bind_template_child (widget_class, GrWindow, details_page);
         gtk_widget_class_bind_template_child (widget_class, GrWindow, edit_page);
         gtk_widget_class_bind_template_child (widget_class, GrWindow, list_page);
+        gtk_widget_class_bind_template_child (widget_class, GrWindow, chef_page);
+        gtk_widget_class_bind_template_child (widget_class, GrWindow, shopping_page);
         gtk_widget_class_bind_template_child (widget_class, GrWindow, search_page);
         gtk_widget_class_bind_template_child (widget_class, GrWindow, cuisines_page);
         gtk_widget_class_bind_template_child (widget_class, GrWindow, cuisine_page);
-        gtk_widget_class_bind_template_child (widget_class, GrWindow, ingredients_page);
+        gtk_widget_class_bind_template_child (widget_class, GrWindow, undo_revealer);
+        gtk_widget_class_bind_template_child (widget_class, GrWindow, undo_label);
 
         gtk_widget_class_bind_template_callback (widget_class, new_recipe);
         gtk_widget_class_bind_template_callback (widget_class, go_back);
@@ -381,8 +469,12 @@ gr_window_class_init (GrWindowClass *klass)
         gtk_widget_class_bind_template_callback (widget_class, hide_or_show_header_end_stack);
         gtk_widget_class_bind_template_callback (widget_class, search_changed);
         gtk_widget_class_bind_template_callback (widget_class, stop_search);
+        gtk_widget_class_bind_template_callback (widget_class, search_mode_changed);
         gtk_widget_class_bind_template_callback (widget_class, window_keypress_handler);
         gtk_widget_class_bind_template_callback (widget_class, window_mapped_handler);
+        gtk_widget_class_bind_template_callback (widget_class, do_undo);
+        gtk_widget_class_bind_template_callback (widget_class, close_undo);
+        gtk_widget_class_bind_template_callback (widget_class, shopping_title_changed);
 }
 
 static void
@@ -416,8 +508,9 @@ gr_window_show_recipe (GrWindow *window,
         gr_details_page_set_recipe (GR_DETAILS_PAGE (window->details_page), recipe);
 
         update_cooking_button (window, gr_details_page_is_cooking (GR_DETAILS_PAGE (window->details_page)));
-
+        g_signal_handlers_block_by_func (window->search_bar, search_mode_changed, window);
         gtk_search_bar_set_search_mode (GTK_SEARCH_BAR (window->search_bar), FALSE);
+        g_signal_handlers_unblock_by_func (window->search_bar, search_mode_changed, window);
 
         gtk_header_bar_set_title (GTK_HEADER_BAR (window->header), gr_recipe_get_translated_name (recipe));
 
@@ -447,11 +540,11 @@ gr_window_edit_recipe (GrWindow *window,
 
 static void
 done_cb (GrRecipeImporter *importer,
-         GrRecipe         *recipe,
+         GList            *recipes,
          GrWindow         *window)
 {
-        if (recipe)
-                gr_window_show_recipe (window, recipe);
+        if (recipes)
+                gr_window_show_list (window, _("Imported Recipes"), recipes);
 }
 
 static void
@@ -527,7 +620,7 @@ gr_window_show_chef (GrWindow *window,
 
         save_back_entry (window);
 
-        gr_list_page_populate_from_chef (GR_LIST_PAGE (window->list_page), chef);
+        gr_list_page_populate_from_chef (GR_LIST_PAGE (window->chef_page), chef);
 
         title = g_strdup_printf (_("Chefs: %s"), gr_chef_get_fullname (chef));
         gtk_header_bar_set_title (GTK_HEADER_BAR (window->header), title);
@@ -536,7 +629,7 @@ gr_window_show_chef (GrWindow *window,
         gtk_stack_set_visible_child_name (GTK_STACK (window->header_title_stack), "title");
         gtk_stack_set_visible_child_name (GTK_STACK (window->header_end_stack), "list");
 
-        gtk_stack_set_visible_child_name (GTK_STACK (window->main_stack), "list");
+        gtk_stack_set_visible_child_name (GTK_STACK (window->main_stack), "chef");
 }
 
 void
@@ -555,6 +648,39 @@ gr_window_show_favorites (GrWindow *window)
         gtk_stack_set_visible_child_name (GTK_STACK (window->main_stack), "list");
 }
 
+void
+gr_window_show_list (GrWindow   *window,
+                     const char *title,
+                     GList      *recipes)
+{
+        save_back_entry (window);
+
+        gr_list_page_populate_from_list (GR_LIST_PAGE (window->list_page), recipes);
+
+        gtk_header_bar_set_title (GTK_HEADER_BAR (window->header), title);
+
+        gtk_stack_set_visible_child_name (GTK_STACK (window->header_start_stack), "back");
+        gtk_stack_set_visible_child_name (GTK_STACK (window->header_title_stack), "title");
+        gtk_stack_set_visible_child_name (GTK_STACK (window->header_end_stack), "list");
+
+        gtk_stack_set_visible_child_name (GTK_STACK (window->main_stack), "list");
+}
+
+void
+gr_window_show_shopping (GrWindow *window)
+{
+        save_back_entry (window);
+
+        gtk_header_bar_set_title (GTK_HEADER_BAR (window->header), _("Cook it later"));
+
+        gtk_stack_set_visible_child_name (GTK_STACK (window->header_start_stack), "back");
+        gtk_stack_set_visible_child_name (GTK_STACK (window->header_title_stack), "title");
+        gtk_stack_set_visible_child_name (GTK_STACK (window->header_end_stack), "list");
+
+        gtk_stack_set_visible_child_name (GTK_STACK (window->main_stack), "shopping");
+        gr_shopping_page_populate (GR_SHOPPING_PAGE (window->shopping_page));
+}
+
 void
 gr_window_show_myself (GrWindow *window)
 {
@@ -568,7 +694,7 @@ gr_window_show_myself (GrWindow *window)
 
         name = gr_recipe_store_get_user_key (store);
         chef = gr_recipe_store_get_chef (store, name);
-        gr_list_page_populate_from_chef (GR_LIST_PAGE (window->list_page), chef);
+        gr_list_page_populate_from_chef (GR_LIST_PAGE (window->chef_page), chef);
 
         gtk_header_bar_set_title (GTK_HEADER_BAR (window->header ), _("My own recipes"));
 
@@ -576,7 +702,7 @@ gr_window_show_myself (GrWindow *window)
         gtk_stack_set_visible_child_name (GTK_STACK (window->header_title_stack), "title");
         gtk_stack_set_visible_child_name (GTK_STACK (window->header_end_stack), "list");
 
-        gtk_stack_set_visible_child_name (GTK_STACK (window->main_stack), "list");
+        gtk_stack_set_visible_child_name (GTK_STACK (window->main_stack), "chef");
 }
 
 void
